package com.huiju.estateagents.eContract.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.eContract.entity.TaCompanySeal;
import com.huiju.estateagents.eContract.service.ITaCompanySealService;
import com.huiju.estateagents.config.FadadaProperties;
import com.huiju.estateagents.sample.entity.TaH5Demand;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import com.huiju.estateagents.eContract.service.ITaCompanyService;
import com.huiju.estateagents.eContract.entity.TaCompany;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 企业认证表  前端控制器
 * </p>
 *
 * @author fxf
 * @since 2020-03-26
 */
@RestController
@RequestMapping("/api")
public class TaCompanyController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaCompanyController.class);

    @Autowired
    private FadadaProperties fadadaProperties;

    @Autowired
    public ITaCompanyService iTaCompanyService;

    @Autowired
    private ITaCompanySealService iTaCompanySealService;

    /**
     * 条件查询企业列表
     *
     * @param pageNum
     * @param pageSize
     * @param companyName
     * @param companyCode
     * @param certifiedStatus
     * @return
     */
    @RequestMapping(value = "/channel/contract/listCompany", method = RequestMethod.GET)
    public ResponseBean listCompany(@RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                   @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                                   String companyName, String companyCode, String certifiedStatus) {
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
            IPage<TaCompany> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaCompany> queryWrapper = new QueryWrapper<>();
            queryWrapper.like(StringUtils.isNotBlank(companyName), "company_name", companyName);
            queryWrapper.like(StringUtils.isNotBlank(companyCode), "company_code", companyCode);
            queryWrapper.eq(StringUtils.isNotBlank(certifiedStatus), "certified_status", certifiedStatus);
            queryWrapper.eq("status", CommConstant.STATUS_NORMAL);
            queryWrapper.orderByDesc("create_date");

            IPage<TaCompany> result = iTaCompanyService.page(pg, queryWrapper);
            responseBean.addSuccess(result);
        } catch (Exception e) {
            logger.error("listCompany -=- {}", e);
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 条件查询企业列表
     *
     * @param pageNum
     * @param pageSize
     * @param companyName
     * @return
     */
    @RequestMapping(value = "/admin/taCompanyList", method = RequestMethod.GET)
    public ResponseBean companyList(@RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                   @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                                   String companyName, String companyCode) {
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
            IPage<TaCompany> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaCompany> queryWrapper = new QueryWrapper<>();
            queryWrapper.like(StringUtils.isNotBlank(companyName), "company_name", companyName);
            queryWrapper.like(StringUtils.isNotBlank(companyCode), "company_code", companyCode);
            queryWrapper.eq("status", CommConstant.STATUS_NORMAL);
            queryWrapper.orderByDesc("create_date");

            IPage<TaCompany> result = iTaCompanyService.page(pg, queryWrapper);
            List<TaCompany> taCompanyList = result.getRecords();
            for (TaCompany taCompany : taCompanyList){
                QueryWrapper<TaCompanySeal> sealQueryWrapper = new QueryWrapper<>();
                sealQueryWrapper.eq("company_id", taCompany.getCompanyId());
                taCompany.setSealNum(iTaCompanySealService.count(sealQueryWrapper));
            }
            responseBean.addSuccess(result);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taCompanyList -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 条件查询企业列表
     *
     * @param pageNum
     * @param pageSize
     * @param companyName
     * @return
     */
    @RequestMapping(value = "/channel/taCompanyList", method = RequestMethod.GET)
    public ResponseBean channelCompanyList(@RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                           @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                                           @RequestParam(value = "companyName") String companyName,
                                           @RequestParam(value = "companyCode")String companyCode,
                                           @RequestParam(value = "certifiedStatus")String certifiedStatus,
                                           @RequestParam(value = "authorizeStatus")String authorizeStatus) {
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
            IPage<TaCompany> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaCompany> queryWrapper = new QueryWrapper<>();
            queryWrapper.like(StringUtils.isNotBlank(companyName), "company_name", companyName);
            queryWrapper.like(StringUtils.isNotBlank(companyCode), "company_code", companyCode);
            queryWrapper.eq(StringUtils.isNotBlank(certifiedStatus), "certified_status", certifiedStatus);
            queryWrapper.eq(StringUtils.isNotBlank(authorizeStatus), "authorize_status", authorizeStatus);
            queryWrapper.eq("status", CommConstant.STATUS_NORMAL);
            queryWrapper.orderByDesc("create_date");

            IPage<TaCompany> result = iTaCompanyService.page(pg, queryWrapper);
            List<TaCompany> taCompanyList = result.getRecords();
            for (TaCompany taCompany : taCompanyList){
                QueryWrapper<TaCompanySeal> sealQueryWrapper = new QueryWrapper<>();
                sealQueryWrapper.eq("company_id", taCompany.getCompanyId());
                taCompany.setSealNum(iTaCompanySealService.count(sealQueryWrapper));
            }
            responseBean.addSuccess(result);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taCompanyList -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     *
     * @param taCompany 实体对象
     * @return
     */
    @RequestMapping(value = "/taCompany", method = RequestMethod.POST)
    public ResponseBean taCompanyAdd(@RequestBody TaCompany taCompany) {
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaCompanyService.save(taCompany)) {
                responseBean.addSuccess(taCompany);
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taCompanyAdd -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     *
     * @param id 实体ID
     */
    @ResponseBody
    @RequestMapping(value = "/taCompany/{id}", method = RequestMethod.DELETE)
    public ResponseBean taCompanyDelete(@PathVariable Integer id) {
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaCompanyService.removeById(id)) {
                responseBean.addSuccess("success");
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taCompanyDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     *
     * @param id        实体ID
     * @param taCompany 实体对象
     * @return
     */
    @RequestMapping(value = "/taCompany/{id}", method = RequestMethod.PUT)
    public ResponseBean taCompanyUpdate(@PathVariable Integer id,
                                        @RequestBody TaCompany taCompany) {
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaCompanyService.updateById(taCompany)) {
                responseBean.addSuccess(taCompany);
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taCompanyUpdate -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     *
     * @param id 实体ID
     */
    @RequestMapping(value = "/channel/taCompany/{id}", method = RequestMethod.GET)
    public ResponseBean taCompanyGet(@PathVariable Integer id) {
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaCompanyService.getById(id));
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taCompanyDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 条件查询企业列表
     *
     * @param pageNum
     * @param pageSize
     * @param companyName
     * @param companyCode
     * @param status
     * @return
     */
    @RequestMapping(value = "/channel/company/list", method = RequestMethod.GET)
    public ResponseBean getCompanyList(@RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                       @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                                       @RequestParam(value = "companyName", required = false) String companyName,
                                       @RequestParam(value = "companyCode", required = false) String companyCode,
                                       @RequestParam(value = "certifiedStatus", required = false) String status) {
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
            IPage<TaCompany> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaCompany> queryWrapper = new QueryWrapper<>();
            queryWrapper.like(StringUtils.isNotBlank(companyName), "company_name", companyName);
            queryWrapper.like(StringUtils.isNotBlank(companyCode), "company_code", companyCode);
            queryWrapper.eq(StringUtils.isNotBlank(status), "status", status);
            queryWrapper.gt("status", CommConstant.STATUS_DELETE);
            queryWrapper.orderByDesc("create_date");

            IPage<TaCompany> result = iTaCompanyService.page(pg, queryWrapper);
            responseBean.addSuccess(result);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taCompanyList -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 第一次自动创建企业信息
     */
    @RequestMapping(value = "/channel/company/add", method = RequestMethod.POST)
    public ResponseBean addCompanyFirstInfo() {
        ResponseBean responseBean = new ResponseBean();
        try {
            TaCompany taCompany = iTaCompanyService.newFirstCompanyInfo();
            responseBean.addSuccess(taCompany);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taCompanyList -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     *
     * @param id 实体ID
     */
    @RequestMapping(value = "/channel/company/{id}", method = RequestMethod.GET)
    public ResponseBean getCompanyInfo(@PathVariable Integer id) {
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaCompanyService.getById(id));
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taCompanyDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id更新数据
     *
     * @param id 实体ID
     */
    @RequestMapping(value = "/channel/company/{id}", method = RequestMethod.PUT)
    public ResponseBean updateCompanyInfo(@PathVariable Integer id,@RequestBody TaCompany taCompany) {
        ResponseBean responseBean = new ResponseBean();
        try {
            taCompany.setCompanyId(id);
            TaCompany newCompany = iTaCompanyService.updateCompanyInfo(taCompany);
            responseBean.addSuccess(newCompany);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taCompanyDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id去法大大那边拿最新数据然后更新
     *
     * @param id 实体ID
     */
    @RequestMapping(value = "/channel/company/fdd/{id}", method = RequestMethod.GET)
    public ResponseBean updateCompanyInfoByFdd(@PathVariable Integer id) {
        ResponseBean responseBean = new ResponseBean();
        try {
            TaCompany newCompany = iTaCompanyService.updateCompanyInfoByFdd(id);
            responseBean.addSuccess(newCompany);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taCompanyDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id更新数据
     *
     * @param
     */
    @RequestMapping(value = "/channel/company/delete", method = RequestMethod.DELETE)
    public ResponseBean updateCompanyInfo(@RequestBody String ids) {
        ResponseBean responseBean = new ResponseBean();
        try {
            int successNum = 0;
            int failNum = 0;
            JSONArray jsonArray = JSONObject.parseObject(ids).getJSONArray("ids");
            for (Object id : jsonArray) {
                TaCompany company = iTaCompanyService.getById((Integer) id);
                if (CommConstant.STATUS_UNACCALIMED.equals(company.getStatus())){
                    company.setStatus(CommConstant.STATUS_DELETE);
                    iTaCompanyService.updateById(company);
                    successNum++;
                }else{
                    failNum++;
                }
            }
            Map<String,Object> map = new HashMap<>();
            map.put("successNum",successNum);
            map.put("failNum",failNum);
            responseBean.addSuccess(map);
        }catch (Exception e){
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 企业认证回调函数
     *
     */
    @RequestMapping(value = "/fadd/company", method = RequestMethod.POST)
    public void faddCompanyNitify(@RequestParam String companyName,
                                  @RequestParam String transactionNo,
                                  @RequestParam String authenticationType,
                                  @RequestParam String status,
                                  HttpServletRequest request) {
        logger.info("企业认证回调函数参数:公司名称{},交易号{}，认证类型{}，状态，{}",companyName,transactionNo,authenticationType,status);
        QueryWrapper<TaCompany> taCompanyQueryWrapper = new QueryWrapper<>();
        taCompanyQueryWrapper.eq("certified_serial_number",transactionNo);
        TaCompany taCompany = iTaCompanyService.getOne(taCompanyQueryWrapper);
        taCompany.setStatus(Integer.valueOf(status));
        iTaCompanyService.updateById(taCompany);
    }

    /**
     * 自动签章授权
     */
    public ResponseBean autoAuthCompanySeal(Integer companyId, HttpServletRequest request){
        return null;
    }
}
